try:
    from Element_Stiffness import *
    from Assemble_Stiffness import *
    from Meshing import *
    from Output import *
    from Plotting import *
    from Shell_Calculation import *
except:
    from KIB_LAP.Scheibe.Element_Stiffness import *
    from KIB_LAP.Scheibe.Assemble_Stiffness import *
    from KIB_LAP.Scheibe.Meshing import *
    from KIB_LAP.Scheibe.Output import *
    from KIB_LAP.Scheibe.Plotting import *
    from KIB_LAP.Scheibe.Shell_Calculation import *